/*
 * Copyright (c) 2024 ShinGeTsu Meter.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 */
#pragma once

// GPOIO
#define LED_PIN 25

// ロータリーエンコーダ
#define TA_RE1 27
#define TA_RE2 28

// ボタン
#define BTN1           5
#define BTN2           4
#define BTN3           3
#define BTN4           2
#define BTN5           13
#define BTN6           12
#define BTN7           11
#define BTN8           10
#define BTN9           9
#define BTN10          8
#define BTN11          7
#define BTN12          6
#define BUTTON_SET     BTN1     // TAセット
#define BUTTON_AUTO    BTN2     // 自動モード切替
#define BUTTON_COUNT   BTN3     // TAカウントモード切替
#define BUTTON_POWER   BTN4     // 電源表示
#define BUTTON_VIEW    BTN5     // 表示モード切替
#define BUTTON_CLEAR   BTN6     // TAカウンタークリア
#define BUTTON_BACK    BTN7     // TAカウンターを履歴から戻す
#define BUTTON_FORWARD BTN8     // 戻したTAカウンターを進める
#define BUTTON_SET2    BTN9     // TAセットその２
#define BUTTON_OPT1    BTN10    // 未使用
#define BUTTON_OPT2    BTN11    // 未使用
#define BUTTON_OPT3    BTN12    // 未使用

// 内蔵ADC
#define VSYS_ADC_PIN 29
#define VSYS_ADC_NO  3

// USB接続
#define VBUS_SENSE_PIN 24

// ロータリーエンコーダ移動方向判定用
#define ROTARY_DIRECTION \
    {0, 1, -1, 0, -1, 0, 0, 1, 1, 0, 0, -1, 0, -1, 1, 0}

// 時間
#define SAMPLING_uS        (50 * 1000)      // サンプリング間隔
#define UPDATE_uS          (500 * 1000)     // LCD更新間隔・・・0.5秒
#define EVENT_WAIT_uS      (10 * 1000)      // 同じイベントの連続実行を抑止する時間
#define POWER_VIEW_WAIT_uS (2000 * 1000)    // 起動時に電源状態を表示する時間・・・2秒

// 自動セット待ち時間
#define AUTO_SET_WAIT_uS  (500 * 1000)                    // デフォルト
#define AUTO_SET_ADJ_STEP ((100 * 1000) / SAMPLING_uS)    // 100ミリ秒のサンプリング数

// 自動セットブロック
#define TA_HIST_uS   (1000 * 1000)                 // TA履歴を保存期間
#define TA_HIST_SIZE (TA_HIST_uS / SAMPLING_uS)    // TA履歴サイズ

// TAカウンタ
#define COUNT_HIST_SIZE 20    // 履歴サイズ

// 感度
#define SENS_AVE_N         8                            // 計算に使用する履歴数
#define GET_SENS(RAW_GAIN) (RAW_GAIN * 0.350 - 1.28)    // 実回路の電圧や抵抗値によって計算式は異なる

// TA値計算用
#define CONST_V 2.5f        // ブリッジの定電圧値
#define CONST_R 21250.0f    // ブリッジの基準抵抗値

// TA値
#define TA_MAX         6.5f       // TA最大値
#define TA_MIN         0.9412f    // TA最小値
#define TA_PAUSE_RANGE 0.800f     // 履歴中の範囲がこれより大きい場合、カウント停止
#define PC_TA_MAX      6.4f       // PCが缶を持っていないと判定する閾値

// 感度補正
#define CORRECT_TA_BASE 2.4f     // 補正を等倍とするTA
#define CORRECT_TA_MIN  2.3f     // 補正範囲の最小
#define CORRECT_TA_MAX  5.5f     // 補正範囲の最大
#define CORRECT_TA_TH   4.3f     // 補正を緩くする境界値
#define CORRECT_TA_RATE 0.83f    // 補正を緩くする割合

// メーター目盛   左端=0, セット位置=15,  右端=40
#define METER_SCALE_SET 15.0f    // セット位置
#define METER_SCALE_MAX 40.0f    // 最大目盛
#define METER_ADJ       1.00f    // 電流計の補正値 個体差があるので実機に合わせて調整する 0.95 ~ 1.05
// #define METER_ADJ 0.95f

// 表示モード
typedef enum {
    VIEW_NORMAL = 0,    // 通常表示
    VIEW_TA,            // 詳細表示(TA)
    VIEW_ADC,           // 詳細表示(電圧)
    VIEW_CONF,          // 設定モード
} view_t;
